# sugar-lint: disable

import os
import sys
import shutil
import logging
import unittest
import subprocess
from os.path import dirname, join, exists, abspath

from libmace import env, files as files_, providers as providers_


root = abspath(dirname(__file__))
tmproot = join(root, '.tmp')
tmpdir = None
_environ = os.environ.copy()


def main():
    shutil.rmtree(tmproot, ignore_errors=True)
    unittest.main()


class Test(unittest.TestCase):

    def setUp(self):
        global tmpdir
        tmpdir = join(tmproot, '.'.join(self.id().split('.')[1:]))
        shutil.rmtree(tmpdir, ignore_errors=True)
        os.makedirs(tmpdir)
        os.chdir(tmpdir)

        logfile = tmpdir + '.log'
        if exists(logfile):
            os.unlink(logfile)

        self._logfile = file(logfile + '.out', 'a')
        sys.stdout = sys.stderr = self._logfile

        env.include.value = None
        env.exclude.value = None
        env.debug.value = True
        for handler in logging.getLogger().handlers:
            logging.getLogger().removeHandler(handler)
        logging.basicConfig(level=logging.DEBUG, filename=logfile)

        self._overriden = []
        providers_.PROVIDERS_DIR = join(root, 'data/providers')
        providers_.BaseProvider.apply = lambda * args: None
        env.source.value = tmpdir
        env.output.value = tmpdir
        env.distributor_id.value = None
        env.release.value = None

        def chown(*args):
            self.chown.append(args)

        self.chown = []
        files_.chown = chown

        stat = os.stat('.')
        files_.uid_by_name = lambda x: stat.st_uid
        files_.gid_by_name = lambda x: stat.st_gid

        providers_.BaseProvider.processed_files = []
        providers_.BaseProvider.applied_files = []

        os.environ = _environ.copy()

    def tearDown(self):
        while self._overriden:
            mod, name, old_handler = self._overriden.pop(0)
            setattr(mod, name, old_handler)

        self._logfile.flush()

    def override(self, mod, name, new_handler):
        self._overriden.append((mod, name, getattr(mod, name)))
        setattr(mod, name, new_handler)

    def touch(self, *files):
        for i in files:
            if isinstance(i, str):
                if i.endswith(os.sep):
                    i = i + '.stamp'
                path = i
                if exists(path):
                    content = file(path).read()
                else:
                    content = i
            else:
                path, content = i
                if isinstance(content, list):
                    content = '\n'.join(content)
            path = join(tmpdir, path)

            if not exists(dirname(path)):
                os.makedirs(dirname(path))
            if exists(path):
                os.unlink(path)

            f = file(path, 'w')
            f.write(str(content))
            f.close()


class Provider(providers_.BaseProvider):

    distro = {}
    comment = '#'
    name = 'test'
    provides = ['test']
    service = None
    confiles = [None]
    confpath = lambda self, x: '/etc/test.conf'
    ensure = lambda * args: None
    apply = lambda * args: None
    activate = lambda * args: None
    confpath_header = lambda * args: []
    confile_header = lambda * args: []
    footer = lambda * args: []
    confile_footer = lambda * args: []
    export = lambda * args: None
    convert = lambda self, confile, lines: lines
    get_depends = lambda * args: []

    def __init__(self, name='test'):
        self.name = name
