#!/usr/bin/env python

# sugar-lint: disable

import os
import sys
from os.path import exists, relpath, abspath

sys.path.insert(0, '..')
import tests
from tests import Test, main, Provider
from libmace import util, providers


class ProviderTest(Test):

    def test_deactivate_WaitUntilStop(self):
        cmd = []
        self.override(util, 'call', lambda *args, **kwwards: cmd.append(args))
        provider = _Provider(2, 10)
        provider.deactivate()
        self.assertEqual([('/etc/init.d/foo stop',)], cmd)
        self.assertEqual(0, provider.tries)

    def test_deactivate_FailIfUnstoppable(self):
        cmd = []
        self.override(util, 'call', lambda *args, **kwwards: cmd.append(args))
        provider = _Provider(10, 2)
        self.assertRaises(RuntimeError, provider.deactivate)
        self.assertEqual([('/etc/init.d/foo stop',)], cmd)
        self.assertEqual(8, provider.tries)


class _Provider(Provider):

    service = 'foo'

    def __init__(self, tries=10, max_tries=10):
        providers._STOP_TRIES = max_tries
        Provider.__init__(self)
        self.tries = tries

    def is_active(self):
        self.tries -= 1
        return self.tries > 0


if __name__ == '__main__':
    main()
