#!/usr/bin/env python

# pep8: ignore=E501
# pylint: disable-msg=C0301

import os
import sys
from os.path import exists

sys.path.insert(0, '..')
from tests import Test, main
from tests.providers import iptables
from libmace import files, lsb_release


class ProviderIPTablesTest(Test):

    def setUp(self):
        Test.setUp(self)
        files.uid_by_name = hash
        files.gid_by_name = hash

    def test_CreateServiceOnUbuntu(self):
        self.override(lsb_release, 'distributor_id', lambda: 'Ubuntu')
        self.touch('iptables.d/filter/0.conf')
        provider = iptables.Provider()
        sources, envars = files.load('.', [provider])
        files.commit('.', 'dst', sources, envars)
        assert exists('dst/etc/init.d/iptables')
        self.assertEqual(0755, os.stat('dst/etc/init.d/iptables').st_mode & 0777)
        assert file('dst/etc/init.d/iptables').readline() == '#!/bin/sh\n'
        assert exists('dst/etc/iptables.rules')

    def test_EndWithCommit(self):
        self.override(lsb_release, 'distributor_id', lambda: 'Ubuntu')

        self.touch(
                'iptables.d/filter/0.conf',
                'iptables.d/nat/0.conf',
                'iptables.d/mangle/0.conf',
                )

        provider = iptables.Provider()
        sources, envars = files.load('.', [provider])
        files.commit('.', 'dst', sources, envars)
        self.assertEqual("""\
# WARNING: File was generate by mace utility, do not edit it.
#          Change configuration sources and run mace.

*mangle
iptables.d/mangle/0.conf
COMMIT
*nat
iptables.d/nat/0.conf
COMMIT
*filter
iptables.d/filter/0.conf
COMMIT
""", file('dst/etc/iptables.rules').read())


if __name__ == '__main__':
    main()
