#!/usr/bin/env python

# sugar-lint: disable

import os
import sys
from os.path import exists, lexists, islink

sys.path.insert(0, '..')
import tests
from tests import Test, main
from providers import munin_node
from libmace.files import Transaction
from libmace import lsb_release


_PLUGINS_DIR = munin_node._PLUGINS_DIR


class ProviderSquidTest(Test):

    def setUp(self):
        Test.setUp(self)
        munin_node._PLUGINS_DIR = tests.tmpdir + '/plugins'
        os.makedirs(munin_node._PLUGINS_DIR)
        os.makedirs('./' + munin_node._PLUGINS_DIR)
        self.override(lsb_release, 'distributor_id', lambda: 'Fedora')

    def tearDown(self):
        munin_node._PLUGINS_DIR = _PLUGINS_DIR
        Test.tearDown(self)

    def test_convert_EscapeAllowDots(self):
        provider = munin_node.Provider()
        self.assertEqual(
                ['^1.2.3.4$'],
                provider.convert(None, ['^1.2.3.4$']))
        self.assertEqual(
                ['allow ^1\\.2\\.3\\.4$'],
                provider.convert(None, ['allow ^1.2.3.4$']))
        self.assertEqual(
                ['allow 1.2\\.3.4'],
                provider.convert(None, ['allow 1.2\\.3.4']))

    def test_convert_ResolveAllowHosts(self):
        provider = munin_node.Provider()
        self.assertEqual(
                ['^localhost$'],
                provider.convert(None, ['^localhost$']))
        self.assertEqual(
                ['allow ^127\\.0\\.0\\.1$'],
                provider.convert(None, ['allow ^localhost$']))

    def test_RemoveExistedPlugins(self):
        tran = Transaction('.')
        provider = munin_node.Provider()

        self.touch('plugins/foo')
        provider.write_config(tran, None, [])
        assert not tran.empty
        assert not exists('plugins/foo')

        tran.rollback()
        assert exists('plugins/foo')

    def test_Plugins(self):
        tran = Transaction('.')
        provider = munin_node.Provider()

        provider.write_config(tran, None, ['foo', 'bar_ test'])
        assert not tran.empty

        path = './%s/foo' % munin_node._PLUGINS_DIR
        assert islink(path)
        self.assertEqual(provider.plugin_path('foo'), os.readlink(path))

        path = './%s/bar_test' % munin_node._PLUGINS_DIR
        assert islink(path)
        self.assertEqual(provider.plugin_path('bar_'), os.readlink(path))

        tran.rollback()
        assert not lexists('./%s/foo' % munin_node._PLUGINS_DIR)
        assert not lexists('./%s/bar_test' % munin_node._PLUGINS_DIR)


if __name__ == '__main__':
    main()
