#!/usr/bin/env python

# sugar-lint: disable

import os
import sys
from os.path import exists

sys.path.insert(0, '..')
from tests import Test, main
from providers import squid
from libmace import env, util, files, lsb_release, distro
from libmace.files import Transaction


class ProviderSquidTest(Test):

    def setUp(self):
        Test.setUp(self)
        files.uid_by_name = lambda x: hash(x)
        files.gid_by_name = lambda x: hash(x)
        self.override(lsb_release, 'distributor_id', lambda: 'Fedora')
        self.override(distro, 'resolve_package', lambda *args: True)

    def test_ensure_CreateSwap(self):

        class Conftext(object):

            def by_confile(self, filename):
                return None

        provider = squid.Provider()
        tran = Transaction('.')

        cmd = []
        self.override(util, 'assert_call', lambda x: cmd.extend(x))

        tran.ensure_file('etc/squid/squid.conf',
                'cache_dir type /swap\n' + \
                'cache_effective_user foo\n' + \
                'cache_effective_group bar')

        provider.ensure(Conftext(), tran)
        assert not tran.empty
        self.assertEqual(['squid', '-z'], cmd)
        assert exists('swap')
        assert ('./swap', hash('foo'), hash('bar')) in self.chown

    def test_enusre_Logs(self):

        class Conftext(object):

            def by_confile(self, filename):
                return None

        provider = squid.Provider()
        tran = Transaction('.')

        self.override(util, 'assert_call', lambda x: None)

        tran.ensure_file('etc/squid/squid.conf',
                'cache_dir type /swap\n' + \
                'access_log log1/foo\n' + \
                'cache_log log2/bar\n' + \
                'cache_store_log log3/test')
        self.touch('swap/')

        provider.ensure(Conftext(), tran)
        assert not tran.empty
        assert exists('log1')
        assert exists('log2')
        assert exists('log3')
        assert ('./log1', hash('squid'), hash('squid')) in self.chown
        assert ('./log2', hash('squid'), hash('squid')) in self.chown
        assert ('./log3', hash('squid'), hash('squid')) in self.chown

    def test_ensure_confile_footer_squidGuard(self):
        provider = squid.Provider()

        self.touch('squid.d/conf/0.conf')
        sources, __ = files.load('.', [provider])
        assert not sources[provider].by_confile('filter')
        assert not provider.footer(provider.confpath('conf'), sources[provider])

        self.touch('squid.d/filter/1.conf')
        sources, __ = files.load('.', [provider])
        assert sources[provider].by_confile('filter')
        print provider.footer(provider.confpath('conf'), sources[provider])
        assert 'url_rewrite_program /usr/bin/squidGuard -c /etc/squid/squidGuard.conf' in provider.footer(provider.confpath('conf'), sources[provider])

    def test_ensure_DBFiles(self):

        class Conftext(object):

            def by_confile(self, filename):
                return '/etc/squid/squidGuard.conf' if filename == 'filter' else None

        def call(args):
            if '-C' in args:
                self.touch(args[-1] + '.db')

        self.override(util, 'assert_call', call)

        self.touch(
                ('squid.d/filter/0.conf', [
                    'dbhome /db',
                    'logdir /log',
                    ]),
                ('squid.d/conf/0.conf', []),
                'dst/db/1.db',
                'dst/db/foo/2.db',
                'dst/db/urls',
                'dst/db/domains',
                'dst/db/domains.db',
                'dst/log/log',
                )
        os.chmod('dst/db/1.db', 0444)
        os.chmod('dst/db/foo/2.db', 0400)
        os.chmod('dst/log/log', 0666)
        os.utime('dst/db/domains', (2, 2))
        os.utime('dst/db/domains.db', (1, 1))

        provider = squid.Provider()
        sources, envars = files.load('.', [provider])
        files.commit('.', 'dst', sources, envars)

        assert ('dst/db/1.db', hash('squid'), hash('squid')) in self.chown
        assert ('dst/db/foo/2.db', hash('squid'), hash('squid')) in self.chown

        assert os.stat('dst/db/1.db').st_mode & 0777 == 0664
        assert os.stat('dst/db/foo/2.db').st_mode & 0777 == 0664

        assert exists('dst/db/urls.db')
        assert os.stat('dst/db/domains.db').st_mtime > os.stat('dst/db/domains').st_mtime

        assert ('dst/log/log', hash('squid'), hash('squid')) in self.chown
        assert os.stat('dst/log/log').st_mode & 0777 == 0640


if __name__ == '__main__':
    main()
